# Generated by Django 3.2.16 on 2022-11-26 18:42

import django.contrib.auth.models
from django.db import migrations
from django.conf import settings


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ("auth", "0012_alter_user_first_name_max_length"),
    ]

    operations = [
        migrations.CreateModel(
            name="DescopeUser",
            fields=[],
            options={
                "proxy": True,
                "indexes": [],
                "constraints": [],
            },
            bases=(settings.AUTH_USER_MODEL,),
            managers=[
                ("objects", django.contrib.auth.models.UserManager()),
            ],
        ),
    ]



import logging

from descope import SESSION_TOKEN_NAME
from django.contrib import auth
from django.core.cache import cache

from . import descope_client
from .settings import IS_STAFF_ROLE, IS_SUPERUSER_ROLE

logger = logging.getLogger(__name__)


class DescopeUser(auth.get_user_model()):
    class Meta:
        proxy = True

    # User is always active since Descope will never issue a token for an
    # inactive user
    is_active = True

    def sync(self, session, refresh):
        self.session_token = session[SESSION_TOKEN_NAME]  # this should always exist
        self.refresh_token = refresh
        self.username = self._me.get("userId")
        self.user = self.username
        self.email = self._me.get("email")
        self.is_staff = descope_client.validate_roles(
            self.session_token, [IS_STAFF_ROLE]
        )
        self.is_superuser = descope_client.validate_roles(
            self.session_token, [IS_SUPERUSER_ROLE]
        )
        self.save()

    def __str__(self):
        return f"DescopeUser {self.username}"

    @property
    def _me(self):
        return cache.get_or_set(
            f"descope_me:{self.username}", lambda: descope_client.me(self.refresh_token)
        )

    def get_username(self):
        return self.username



from django.conf import settings
from django.core.exceptions import ImproperlyConfigured

# renovate depName=@descope/web-component datasource=npm
WEB_COMPONENT_VERSION = getattr(settings, "DESCOPE_WEB_COMPONENT_VERSION", "3.23.1")
WEB_COMPONENT_SRC = getattr(
    settings,
    "DESCOPE_WEB_COMPONENT_SRC",
    "https://unpkg.com/@descope/web-component@{:s}/dist/index.js".format(
        WEB_COMPONENT_VERSION
    ),
)

MANAGEMENT_KEY = getattr(settings, "DESCOPE_MANAGEMENT_KEY", "")
PROJECT_ID = getattr(settings, "DESCOPE_PROJECT_ID", "")
if not PROJECT_ID:
    raise ImproperlyConfigured('"DESCOPE_PROJECT_ID" is required!')

# Role names to create in Descope that will map to User attributes
IS_STAFF_ROLE = getattr(settings, "DESCOPE_IS_STAFF_ROLE", "is_staff")
IS_SUPERUSER_ROLE = getattr(settings, "DESCOPE_IS_SUPERUSER_ROLE", "is_superuser")



import json
import logging
import random
import string

import descope
from descope import (
    REFRESH_SESSION_COOKIE_NAME,
    REFRESH_SESSION_TOKEN_NAME,
    SESSION_COOKIE_NAME,
    SESSION_TOKEN_NAME,
)
from django.contrib.auth.mixins import LoginRequiredMixin
from django.http import JsonResponse
from django.test import TestCase, override_settings
from django.urls import path, reverse
from django.views import View

from . import descope_client, urls

logger = logging.getLogger(__name__)


def random_string(N: int) -> str:
    return "".join(random.choices(string.ascii_lowercase + string.digits, k=N))


class TestProtectedView(LoginRequiredMixin, View):
    def get(self, request):
        return JsonResponse({"success": True})


urls.urlpatterns = urls.urlpatterns + [
    path("test", TestProtectedView.as_view(), name="test_protected_view"),
]


@override_settings(ROOT_URLCONF=urls)
class StoreJwtTestCase(TestCase):
    delivery_method = descope.DeliveryMethod.EMAIL
    login_id = f"test+{random_string(8)}@test.internal"
    token: dict

    def setUp(self) -> None:
        descope_client.mgmt.user.create_test_user(
            self.login_id, role_names=["is_staff", "is_superuser"], verified_email=True
        )
        resp = descope_client.mgmt.user.generate_otp_for_test_user(
            self.delivery_method,
            self.login_id,
        )
        self.token = descope_client.otp.verify_code(
            self.delivery_method, self.login_id, resp.get("code")
        )

    def test_store_jwt(self):
        """Test the store_jwt view"""

        # should fail without a session
        res = self.client.get(reverse("test_protected_view"))
        self.assertNotEqual(res.status_code, 200)

        # lets store the jwt
        res = self.client.post(
            reverse("store_jwt"),
            {
                SESSION_COOKIE_NAME: self.token[SESSION_TOKEN_NAME]["jwt"],
                REFRESH_SESSION_COOKIE_NAME: self.token[REFRESH_SESSION_TOKEN_NAME][
                    "jwt"
                ],
            },
        )
        self.assertEqual(res.status_code, 200)

        debug = json.loads(res.content)
        self.assertEqual(debug["success"], True)
        session = self.client.session
        self.assertEqual(  # ensure the session was updated
            session[SESSION_COOKIE_NAME], self.token[SESSION_TOKEN_NAME]["jwt"]
        )
        self.assertEqual(  # ensure the session was updated
            session[REFRESH_SESSION_COOKIE_NAME],
            self.token[REFRESH_SESSION_TOKEN_NAME]["jwt"],
        )

        # should succeed with a session
        res = self.client.get(reverse("test_protected_view"))
        self.assertEqual(res.status_code, 200)
        self.assertEqual(debug["success"], True)

    def tearDown(self) -> None:
        descope_client.mgmt.user.delete(self.login_id)



import logging
from typing import Any, Union

from descope import REFRESH_SESSION_COOKIE_NAME, SESSION_COOKIE_NAME, SESSION_TOKEN_NAME
from descope.exceptions import AuthException
from django.conf import settings
from django.contrib.auth import logout
from django.contrib.auth.backends import BaseBackend
from django.http import HttpRequest

from . import descope_client
from .models import DescopeUser

logger = logging.getLogger(__name__)


def add_tokens_to_request(session: Any, session_token: str, refresh_token: str):
    session[SESSION_COOKIE_NAME] = session_token
    session[REFRESH_SESSION_COOKIE_NAME] = refresh_token
    session.save()


class DescopeAuthentication(BaseBackend):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    def authenticate(self, request: Union[HttpRequest, None], **kwargs):
        if request is None:
            return None
        session_token = request.session.get(SESSION_COOKIE_NAME, "")
        refresh_token = request.session.get(REFRESH_SESSION_COOKIE_NAME, "")

        logger.debug("Validating (and refreshing) Descope session")
        try:
            validated_session = descope_client.validate_and_refresh_session(
                session_token, refresh_token
            )

        except AuthException as e:
            """
            Ask forgiveness, not permission.
                - Grace Hopper

            This exception will be thrown on every unauthenticated request to
            ensure logging out an invalid user.
            """
            logger.debug(e)
            logout(request)
            return None

        if settings.DEBUG:
            # Contains sensitive information, so only log in DEBUG mode
            logger.debug(validated_session)
        if validated_session:
            username = validated_session[SESSION_TOKEN_NAME]["sub"]
            user, _ = DescopeUser.objects.get_or_create(username=username)
            user.sync(validated_session, refresh_token)
            request.session[SESSION_COOKIE_NAME] = user.session_token["jwt"]
            return user
        else:
            return None



import logging

from descope import REFRESH_SESSION_COOKIE_NAME, SESSION_COOKIE_NAME
from django.http import HttpRequest, HttpResponseBadRequest, JsonResponse
from django.utils.decorators import method_decorator
from django.views import View
from django.views.decorators.cache import never_cache

from .authentication import add_tokens_to_request

logger = logging.getLogger(__name__)


@method_decorator([never_cache], name="dispatch")
class StoreJwt(View):
    def post(self, request: HttpRequest):
        session = request.POST.get(SESSION_COOKIE_NAME)
        refresh = request.COOKIES.get(REFRESH_SESSION_COOKIE_NAME)
        if not refresh:
            refresh = request.POST.get(REFRESH_SESSION_COOKIE_NAME)

        if session and refresh:
            add_tokens_to_request(request.session, session, refresh)
            return JsonResponse({"success": True})

        return HttpResponseBadRequest()
